"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Migration = void 0;
const runTask_1 = require("./tasks/runTask");
const MigrationEmitter_1 = require("./MigrationEmitter");
const utils_1 = require("./utils");
const reporter_1 = require("./reporter");
const VirtualFileSystem_1 = require("./VirtualFileSystem");
class Migration {
    constructor(options) {
        this.transform = (title, pattern, transformFn) => {
            this.runTask({ type: 'transform', title, pattern, fn: transformFn });
        };
        this.rename = (title, pattern, renameFn) => {
            this.runTask({ type: 'rename', title, pattern, fn: renameFn });
        };
        this.remove = (title, pattern, removeFn) => {
            this.runTask({ type: 'remove', title, pattern, fn: removeFn });
        };
        this.create = (title, patternOrCreateFn, createFn) => {
            let task;
            if (utils_1.isPattern(patternOrCreateFn)) {
                if (!createFn) {
                    throw new Error(`When using a pattern for the second argument of the createTask function
You must supply a createFunction as the third argument`);
                }
                task = {
                    type: 'create',
                    title,
                    pattern: patternOrCreateFn,
                    fn: createFn,
                };
            }
            else {
                task = { type: 'create', title, fn: patternOrCreateFn };
            }
            this.runTask(task);
        };
        this.after = (afterFn) => {
            this.afterHooks.push(afterFn);
        };
        this.registerMethods = {
            transform: this.transform,
            rename: this.rename,
            remove: this.remove,
            create: this.create,
            after: this.after,
        };
        this.options = options;
        this.events = new MigrationEmitter_1.MigrationEmitter(this);
        this.fs = new VirtualFileSystem_1.VirtualFileSystem({ cwd: options.cwd });
        this.instructions = [];
        this.afterHooks = [];
    }
    runTask(task) {
        this.events.emit('task-start', { task });
        this.instructions.push(...runTask_1.runSingleTask(task, this));
    }
    getMigrationInstructions() {
        return this.instructions;
    }
    write() {
        this.fs.writeChangesToDisc();
    }
    run() {
        this.write();
        // run after write hooks
        this.afterHooks.forEach((fn) => fn());
    }
    static create(options) {
        const migration = new Migration(options);
        reporter_1.reporter(migration);
        return migration;
    }
}
exports.Migration = Migration;
