"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createReport = exports.formatSingleFileAction = void 0;
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = require("lodash");
const formatSingleFileAction = (fileAction) => {
    switch (fileAction.type) {
        case 'transform': {
            const { newFile } = fileAction;
            return `${chalk_1.default.blue('transform:')} ${newFile.fileName}`;
        }
        case 'create': {
            const { newFile } = fileAction;
            return `${chalk_1.default.green('create:')} ${newFile.fileName}`;
        }
        case 'remove': {
            return `${chalk_1.default.red('remove:')} ${fileAction.file.fileName}`;
        }
        case 'rename': {
            const { originalFile, newFile } = fileAction;
            return `${chalk_1.default.yellow('rename:')} ${originalFile.fileName} -> ${newFile.fileName}`;
        }
        default: {
            // @ts-expect-error ts thinks that task is "never"
            throw new Error(`unknown fileAction type "${fileAction.type}"`);
        }
    }
};
exports.formatSingleFileAction = formatSingleFileAction;
const createReport = (fileActions) => {
    if (lodash_1.isEmpty(fileActions)) {
        return chalk_1.default.blue('🤷‍♂️ the migration passed without changes');
    }
    const output = [];
    for (const [taskTitle, taskFileActions] of Object.entries(lodash_1.groupBy(fileActions, 'task.title'))) {
        output.push(chalk_1.default.underline(chalk_1.default.bold(taskTitle)) +
            '\n' +
            taskFileActions.map(exports.formatSingleFileAction).join('\n'));
    }
    return output.join('\n\n');
};
exports.createReport = createReport;
