"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runMigration = void 0;
const chalk_1 = __importDefault(require("chalk"));
const prompts_1 = __importDefault(require("prompts"));
const Migration_1 = require("./Migration");
const createReport_1 = require("./createReport");
const loadUserMigrationFile_1 = require("./loadUserMigrationFile");
const lodash_1 = require("lodash");
/**
 *
 * @param options.cwd The directory of the project which the migration runs on
 * @param options.migrationFilePath path to the migration file
 * @param options.dry dry run mode
 * @param options.yes do not prompt the user with confirmation
 * Run a migration
 *
 */
const runMigration = async ({ cwd, migrationFilePath, dry, yes, }) => {
    const migration = Migration_1.Migration.create({ cwd });
    await loadUserMigrationFile_1.loadUserMigrationFile(migration, migrationFilePath);
    const fileActions = migration.getMigrationInstructions();
    if (dry) {
        console.log(chalk_1.default.bold('dry-run mode, no files will be modified'));
        console.log();
        console.log(createReport_1.createReport(fileActions));
        process.exit(0);
    }
    console.log();
    console.log(createReport_1.createReport(fileActions));
    if (lodash_1.isEmpty(fileActions)) {
        process.exit(0);
    }
    if (!yes) {
        // space the prompt
        console.log();
        const response = await prompts_1.default({
            type: 'confirm',
            name: 'value',
            message: 'Do you want to perform the migration on the above files?',
            initial: true,
        });
        if (!response.value) {
            console.log('');
            console.log(chalk_1.default.red('Migration aborted'));
            process.exit(1);
        }
    }
    migration.run();
    console.log();
    console.log(chalk_1.default.green('The migration was done successfully 🎉'));
};
exports.runMigration = runMigration;
