"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCreateTask = void 0;
const File_1 = require("../File");
const utils_1 = require("../utils");
const lodash_1 = require("lodash");
const runCreateTask = (task, migration) => {
    if (!task.pattern) {
        return [createFile()].filter(utils_1.isTruthy);
    }
    const files = File_1.getFiles(migration.options.cwd, task.pattern, migration);
    const fileResults = files.map(createFile).filter(utils_1.isTruthy);
    return fileResults;
    function createFile(file) {
        migration.events.emit('create-start', { file, task });
        let createdFile = {};
        try {
            if (file) {
                createdFile = task.fn({
                    fileName: file.fileName,
                    source: file.source,
                });
            }
            else {
                // @ts-expect-error we know that in this case, this is an EmptyCreateFn
                createdFile = task.fn();
            }
        }
        catch (error) {
            migration.events.emit('create-fail', {
                file,
                error,
                task,
            });
            return null;
        }
        if (lodash_1.isNull(createdFile)) {
            migration.events.emit('create-success-cancle', { task });
            return null;
        }
        if (!createdFile.fileName) {
            throw new Error('the return value of a create function needs to contain an object with { fileName: <string> }');
        }
        if (lodash_1.isUndefined(createdFile.source)) {
            throw new Error('the return value of a create function needs to contain an object with { source: <string> }');
        }
        const originalFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: createdFile.fileName,
            migration,
        });
        const newFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: createdFile.fileName,
            source: createdFile.source,
            migration,
        });
        const fileAction = {
            newFile,
            type: task.type,
            task,
        };
        if (originalFile.exists) {
            // Add the original File to mark that the file exists
            fileAction.originalFile = originalFile;
            // @ts-expect-error - we know that originalFile exists here
            migration.events.emit('create-success-override', fileAction);
        }
        migration.events.emit('create-success', fileAction);
        migration.fs.writeFileSync(newFile.path, newFile.source);
        return fileAction;
    }
};
exports.runCreateTask = runCreateTask;
