"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runRenameTask = void 0;
const lodash_1 = require("lodash");
const File_1 = require("../File");
const utils_1 = require("../utils");
const runRenameTask = (task, migration) => {
    const files = File_1.getFiles(migration.options.cwd, task.pattern, migration);
    const fileResults = files
        .map((file) => {
        migration.events.emit('rename-start', { file, task });
        let renamedFile;
        try {
            renamedFile = task.fn(file);
        }
        catch (error) {
            migration.events.emit('rename-fail', {
                file,
                error,
                task,
            });
            return null;
        }
        const newFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: renamedFile || file.fileName,
            source: file.source,
            migration,
        });
        const isRenamed = !lodash_1.isEqual(newFile.fileName, file.fileName);
        if (isRenamed) {
            migration.events.emit('rename-success-change', {
                task,
                originalFile: file,
                newFile,
            });
            migration.fs.renameSync(file.path, newFile.path);
            return {
                type: task.type,
                originalFile: file,
                newFile,
                task,
            };
        }
        migration.events.emit('rename-success-noop', {
            task,
            file,
        });
        return null;
    })
        .filter(utils_1.isTruthy);
    return fileResults;
};
exports.runRenameTask = runRenameTask;
