"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runTransformTask = void 0;
const lodash_1 = require("lodash");
const File_1 = require("../File");
const utils_1 = require("../utils");
const runTransformTask = (task, migration) => {
    const files = File_1.getFiles(migration.options.cwd, task.pattern, migration);
    let aborted = false;
    const abort = () => {
        aborted = true;
    };
    let fileResults = [];
    for (let file of files) {
        // We let the user control this callback
        // If it was call, we want to stop looping the files
        if (aborted) {
            break;
        }
        migration.events.emit('transform-start', { file, task });
        let transformedSource;
        try {
            transformedSource = task.fn({
                source: file.source,
                fileName: file.fileName,
                abort,
            });
        }
        catch (error) {
            migration.events.emit('transform-fail', {
                file,
                error,
                task,
            });
            continue;
        }
        let source;
        if (lodash_1.isObject(transformedSource)) {
            source = utils_1.strigifyJson(transformedSource);
        }
        else {
            source = transformedSource;
        }
        const newFile = new File_1.File({
            cwd: migration.options.cwd,
            fileName: file.fileName,
            source,
            migration,
        });
        const isRenamed = !lodash_1.isEqual(newFile.fileName, file.fileName);
        const isModified = isRenamed || !lodash_1.isEqual(newFile.source, file.source);
        if (isModified) {
            const fileAction = {
                originalFile: file,
                newFile,
                type: task.type,
                task,
            };
            migration.events.emit('transform-success-change', fileAction);
            fileResults.push(fileAction);
            continue;
        }
        migration.events.emit('transform-success-noop', {
            task,
            file,
        });
    }
    if (aborted) {
        return [];
    }
    fileResults.forEach((fileAction) => {
        if (fileAction.type !== 'transform') {
            throw new Error('wrong action type');
        }
        migration.fs.removeSync(fileAction.originalFile.path);
        migration.fs.writeFileSync(fileAction.newFile.path, fileAction.newFile.source);
    });
    return fileResults;
};
exports.runTransformTask = runTransformTask;
