"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrationTestkit = exports.createTestkit = void 0;
process.env.NODE_ENV = 'test';
const path_1 = __importDefault(require("path"));
const tempy_1 = __importDefault(require("tempy"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const globby_1 = __importDefault(require("globby"));
const expect_1 = __importDefault(require("expect"));
const Migration_1 = require("../Migration");
const loadUserMigrationFile_1 = require("../loadUserMigrationFile");
/**
 * @param options.migrationFile an absolute path to a migration file or a relative path
 * to the fixtures directory, defaults to migration.ts
 *
 */
const createTestkit = ({ migrationFile = 'migration.ts', } = {}) => {
    return new MigrationTestkit({ migrationFile });
};
exports.createTestkit = createTestkit;
class MigrationTestkit {
    constructor({ migrationFile }) {
        this.migrationFile = migrationFile;
    }
    /**
     * @param options.fixtures an absolute path to a fixtures directory
     * which contains \_\_before__ and \_\_after__ directories
     */
    async run({ fixtures }) {
        if (!fixtures) {
            throw new Error('must provide "fixtures" path');
        }
        if (!path_1.default.isAbsolute(fixtures)) {
            throw new Error(`fixtures path must be an absolute path`);
        }
        if (!fs_extra_1.default.existsSync(fixtures)) {
            throw new Error(`fixture path ${fixtures} doesn't exist`);
        }
        let migrationFile = this.migrationFile;
        // join with fixtures directory in case of a relative path
        if (!path_1.default.isAbsolute(migrationFile)) {
            migrationFile = path_1.default.join(fixtures, migrationFile);
        }
        if (!fs_extra_1.default.existsSync(fixtures)) {
            throw new Error(`migration file ${migrationFile} doesn't exist`);
        }
        const beforeDirectory = path_1.default.join(fixtures, '__before__');
        const afterDirectory = path_1.default.join(fixtures, '__after__');
        if (!fs_extra_1.default.existsSync(beforeDirectory)) {
            throw new Error(`please create a "__before__" directory in ${fixtures}`);
        }
        if (!fs_extra_1.default.existsSync(afterDirectory)) {
            throw new Error(`please create a "__after__" directory in ${afterDirectory}`);
        }
        const workingDir = tempy_1.default.directory();
        const migration = Migration_1.Migration.create({ cwd: workingDir });
        fs_extra_1.default.copySync(beforeDirectory, workingDir);
        await loadUserMigrationFile_1.loadUserMigrationFile(migration, migrationFile);
        migration.run();
        const expectedFiles = globby_1.default.sync('**/*', {
            cwd: afterDirectory,
            gitignore: true,
            dot: true,
            ignore: ['**/node_modules/**'],
        });
        const resultFiles = globby_1.default.sync('**/*', {
            cwd: workingDir,
            gitignore: true,
            dot: true,
            ignore: ['**/node_modules/**'],
        });
        expectedFiles.forEach((fileName) => {
            try {
                expect_1.default(resultFiles).toContain(fileName);
            }
            catch (error) {
                throw new Error(`
Migration file: ${migrationFile}

${error.toString()}`);
            }
            const expectedFilePath = path_1.default.join(afterDirectory, fileName);
            const expectedFileContents = fs_extra_1.default.readFileSync(expectedFilePath, 'utf-8');
            const resultFilePath = path_1.default.join(workingDir, fileName);
            const resultFileContents = fs_extra_1.default.readFileSync(resultFilePath, 'utf-8');
            try {
                expect_1.default(resultFileContents).toBe(expectedFileContents);
            }
            catch (error) {
                throw new Error(`
Migration file: ${migrationFile}
Expected file: ${expectedFilePath}
Recieved file: ${resultFilePath}

${error.toString()}`);
            }
        });
    }
}
exports.MigrationTestkit = MigrationTestkit;
